/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.vew.sim.sim.sender;
 
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Klasse zum Versenden von Daten an bestimmte Objekte. 
 * Sie implentiert das ClientSenderInterface welches die Methoden
 * zum Versenden der Daten bereitstellt.
 * 
 * @version $Revision: 1.3 $ / $Date: 2008/10/31 13:30:33 $ / ($Author: Sans $)
 * 
 * @author Dambach-Werke GmbH
 * @author Stefan Sans
 * 
 */
public class Sender implements ClientSenderInterface
{
  
  /**
   * Debug-Logger für Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

  /**
   * Ist true gesetzt, solage auf Sendesteuerung geartet wird
   */
  private boolean warten = false;

  /**
   * Die datenverteiler-Verbindung
   */
  protected final ClientDavInterface _connection;

  /**
   * Erfolgt das Senden mit dieser Instanz als Quelle?
   */
  protected final boolean _alsQuelle;

  /**
   * Zeitin ms, die max. auf Sendesteuerung gewartet wird
   */
  protected final long _timeout;

  /**
   * Zeitin ms, die max. auf Sendesteuerung gewartet wird
  */

  /**
   * Default-Zeitin ms, die max. auf Sendesteuerung gewartet wird
  */
  public static final long DEFAULT_TIMEOUT = 5000;



  /**
   * Konstruktor der Klasse
   *
   * @param connection Die Datenverteiler-Verbindung,
   * über die gesendet werden soll
   */
  public Sender(ClientDavInterface connection)
  {
    this(connection, DEFAULT_TIMEOUT, false);
  }

  /**
   * Konstruktor der Klasse
   *
   * @param connection Die Datenverteiler-Verbindung,
   * über die gesendet werden soll
   * @param timeout die Zeit in ms, die max. auf Sendesteuerung
   * gewartet weren soll
   */
  public Sender(ClientDavInterface connection, long timeout)
  {
    this(connection, timeout, false);
  }

  /**
   * Konstruktor der Klasse
   *
   * @param connection Die Datenverteiler-Verbindung,
   * über die gesendet werden soll
   * @param alsQuelle soll als Quelle gesendet werden?
   */
  public Sender(ClientDavInterface connection, boolean alsQuelle)
  {
    this(connection, DEFAULT_TIMEOUT, alsQuelle);
  }

  /**
   * Konstruktor der Klasse
   *
   * @param connection Die Datenverteiler-Verbindung,
   * über die gesendet werden soll
   * @param timeout die Zeit in ms, die max. auf Sendesteuerung
   * gewartet weren soll
   * @param alsQuelle soll als Quelle gesendet werden?
   */
  public Sender(ClientDavInterface connection, long timeout,
      boolean alsQuelle)
  {
    _connection = connection;
    _timeout = timeout;
    _alsQuelle = alsQuelle;
  }

  /**
   * Senden von Daten.
   * Die Simulationsvariante ist unspezifiziert
   *
   * @param so das Objekt, für das die Sendeanmeldung erfolgt
   * @param atg die Attributgruppw
   * @param asp der Aspekt
   * @param data der Datensatz
   * @return true, wenn das Senden erfolgreich war, false sonst
   */
  public boolean send(SystemObject so, AttributeGroup atg, Aspect asp, Data data)
  {
    return send(so, atg, asp, DataDescription.NO_SIMULATION_VARIANT_SET, data);
  }

  /**
   * Senden von Daten.
   * Die Simulationsvariante ist spezifiziert
   *
   * @param so das Objekt, für das die Sendeanmeldung erfolgt
   * @param atg die Attributgruppw
   * @param asp der Aspekt
   * @param sv die Simulationsvariante
   * @param data der Datensatz
   * @return true, wenn das Senden erfolgreich war, false sonst
   */
  public boolean send(SystemObject so, AttributeGroup atg, Aspect asp, short sv, Data data)
  {

    DataDescription dd = new DataDescription(atg, asp, sv);
    ResultData resultData = new ResultData(so, dd, _connection.getTime(), data);

    return send(resultData);
  }

  /**
   * Senden von Daten.
   *
   * @param resultData die zu sendenden Daten
   * @return true, wenn das Senden erfolgreich war, false sonst
   */
  public boolean send(ResultData resultData)
  {
    boolean result = true;
    try
    {
      _connection.sendData(resultData);
    }
    catch (DataNotSubscribedException e)
    {
      try
      {
        _connection.subscribeSender(this, resultData.getObject(),
            resultData.getDataDescription(), _alsQuelle ?
                SenderRole.source() : SenderRole.sender());
      }
      catch (OneSubscriptionPerSendData e1)
      {
        // Ist schon subscribed?
        // tue nichts
      }
      try
      {
        _connection.sendData(resultData);
      }
      catch (DataNotSubscribedException e1)
      {
        e1.printStackTrace();
        throw(e1);
      }
      catch (SendSubscriptionNotConfirmed e1)
      {
        // keine Sendesteuerung
        result =  sendData(resultData);
      }
    }
    catch (SendSubscriptionNotConfirmed e)
    {
      // keine Sendesteuerung
      result = sendData(resultData);
    }

    if (!result)
    {
      _debug.fine("Keine Sendesteuerung!");
    }

    return result;
  }

  /**
   * Senden von Daten.
   *
   * Diese nur intern zu benutzende Methode implementiert das Warten auf
   * Sendesteuerung, macht aber slebst keinerlei Sendeanmeldung
   *
   * @param resultData die zu sendenden Daten
   * @return true, wenn das Senden erfolgreich war, false sonst
   */
  protected boolean sendData(ResultData resultData)
  {
    if (_timeout > 0)
    {
      _debug.fine("Warte max. " + _timeout + "ms auf Sendesteuerung");
      synchronized(this)
      {
        warten = true;
        try
        {
          wait(_timeout);
        }
        catch(InterruptedException e)
        {
          // tue nichts
        }
        warten = false;
      }
      try
      {
        _connection.sendData(resultData);
      }
      catch (SendSubscriptionNotConfirmed e)
      {
        //e.printStackTrace();
        return false;
      }
      return true;
    }
    else
    {
      return false;
    }
  }

  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
    _debug.finer("dataRequest() Sender state: " + state);
    if (warten && state == ClientSenderInterface.START_SENDING)
    {
      synchronized(this)
      {
        notify();
      }
    }
  }

  public boolean isRequestSupported(SystemObject so, DataDescription dd)
  {
    return true;
  }

  /**
  * Senden von Daten.
  *
  * Bei Bedarf erfolgt die Sendeanmledung sowie unmittelbar nach dem Senden
  * die Abmeldung
  *
  * @param con die Datenverteiler-Verbindung, die zum Senden benutzt werden soll
  * @param data die zu sendenden Daten
  * @return true, wenn das Senden erfolgreich war, false sonst
  */
  public static boolean send(ClientDavInterface con, ResultData data)
  {
    boolean result;
    Sender sender = new Sender(con);

    try
    {
      result = sender.send(data);
    }
    finally
    {
      try
      {
        con.unsubscribeSender(sender, data.getObject(), data.getDataDescription());
      }
      catch (Exception e)
      {
        // tue nichts
      }
    }

    return result;
  }

  /**
   * Senden von Daten als Sender.
   *
   * Bei Bedarf erfolgt die Sendeanmledung sowie unmittelbar nach dem Senden
   * die Abmeldung. Die Simulationsvariante ist unspezifiziert
   *
   * @param con die Datenverteiler-Verbindung, die zum Senden benutzt werden soll
   *  @param so das Objekt, für das die Sendeanmeldung erfolgt
   * @param atg die Attributgruppw
   * @param asp der Aspekt
   * @param data der Datensatz
   * @return true, wenn das Senden erfolgreich war, false sonst
   */
  public static boolean send(ClientDavInterface con, SystemObject so,
      AttributeGroup atg, Aspect asp, Data data)
  {
    return send(con, so, atg, asp, DataDescription.NO_SIMULATION_VARIANT_SET, data);
  }

  /**
   * Senden von Daten Sender.
   * Bei Bedarf erfolgt die Sendeanmledung sowie unmittelbar nach dem Senden
   * die Abmeldung. Die Simulationsvariante ist zu spezifizieren
   *
   * @param con die Datenverteiler-Verbindung, die zum Senden benutzt werden soll
   * @param so das Objekt, für das die Sendeanmeldung erfolgt
   * @param atg die Attributgruppw
   * @param asp der Aspekt
   * @param sv die Simulationsvariante
   * @param data der Datensatz
   * @return boolean
   */
  public static boolean send(ClientDavInterface con, SystemObject so, 
      AttributeGroup atg, Aspect asp, short sv, Data data)
  {

    DataDescription dd = new DataDescription(atg, asp, sv);
    ResultData resultData = new ResultData(so, dd, con.getTime(), data);
    
    return send(con, resultData);
  }

}
